# content of test_sample.py

import pytest
from folio_uuid import FOLIONamespaces
from folio_uuid import FolioUUID

made_up_okapi_url = "https://okapi.folio.ebsco.com"


def test_deterministic_uuid_generation_basic():
    deterministic_uuid = FolioUUID(made_up_okapi_url, FOLIONamespaces.items, "test")
    assert "ccfe50c7-e610-5584-a826-f918d7c8684d" == str(deterministic_uuid)


def test_deterministic_uuid_generation_sierra_weak_record_key():
    deterministic_uuid = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.items,
        "i3696836",
    )
    assert "da5b73ed-73ff-5d0f-8e11-e902a10dc60a" == str(deterministic_uuid)


def test_deterministic_uuid_generation_integer():
    deterministic_uuid = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.items,
        3696836,
    )
    assert "d0fb4758-e240-55f9-ae0f-fa1a1d693179" == str(deterministic_uuid)


def test_deterministic_uuid_generation_sierra_weak_record_key2():
    deterministic_uuid = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.items,
        "3696836",
    )
    assert deterministic_uuid


def test_deterministic_uuid_generation_empty_legacy_id():
    with pytest.raises(ValueError):
        FolioUUID(
            made_up_okapi_url,
            FOLIONamespaces.items,
            "",
        )


def test_deterministic_uuid_generation_space_legacy_id():
    with pytest.raises(ValueError):
        FolioUUID(
            made_up_okapi_url,
            FOLIONamespaces.items,
            " ",
        )


def test_deterministic_uuid_generation_none_legacy_id():
    with pytest.raises(ValueError):
        FolioUUID(
            made_up_okapi_url,
            FOLIONamespaces.items,
            None,
        )


def test_deterministic_uuid_generation_sierra_strong_record_key():
    deterministic_uuid = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.items,
        "i36968365",
    )
    assert "da5b73ed-73ff-5d0f-8e11-e902a10dc60a" == str(deterministic_uuid)


def test_deterministic_uuid_generation_sierra_strong_record_key_dot():
    deterministic_uuid = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.items,
        ".i36968365",
    )
    assert "da5b73ed-73ff-5d0f-8e11-e902a10dc60a" == str(deterministic_uuid)


def test_deterministic_uuid_generation_holdings():
    deterministic_uuid = FolioUUID(
        "https://okapi-bugfest-juniper.folio.ebsco.com",
        FOLIONamespaces.holdings,
        "000000167",
    )
    assert "a0b4c8a2-01fd-50fd-8158-81bd551412a0" == str(deterministic_uuid)


def test_checkdigit_creation():
    """Translated to python from https://github.com/SydneyUniLibrary/sierra-record-check-digit/blob/master/index-test.js"""
    known_check_digits = [
        [100114, "0"],
        [2539964, "0"],
        [100610, "1"],
        [1655776, "1"],
        [583623, "2"],
        [1629736, "2"],
        [572288, "3"],
        [4093863, "3"],
        [284683, "4"],
        [3898776, "4"],
        [395792, "5"],
        [3040121, "5"],
        [542671, "6"],
        [2626834, "6"],
        [459573, "7"],
        [2699873, "7"],
        [581326, "8"],
        [2054794, "8"],
        [539148, "9"],
        [1203395, "9"],
        [585345, "x"],
        [1562237, "x"],
    ]

    for combo in known_check_digits:
        assert FolioUUID.calculate_sierra_check_digit(combo[0]) == combo[1]


def test_obvious_record_numbers_with_check_digits():
    ids = [
        ["b33846327", "b3384632"],
        ["b47116523@mdill", "b4711652"],
        ["o100007x", "o100007"],
        ["b1125421x", "b1125421"],
        [".b1125421x", "b1125421"],
        ["#b33846327", "#b33846327"],
        ["b", "b"],
        [".b", ".b"],
        ["9999999", "9999999"],
        ["https://id.kb.se/fnrgl", "https://id.kb.se/fnrgl"],
        ["111", "111"],
        ["111111111111", "111111111111"],
        ["b12345", "b12345"],
        ["b4711652@", "b4711652@"],
        ["b4711652@toolong", "b4711652@toolong"],
        [".b12345", ".b12345"],
        [".b4711652@", ".b4711652@"],
        [".b4711652@toolong", ".b4711652@toolong"],
    ]

    for id_combo in ids:
        assert FolioUUID.clean_iii_identifiers(id_combo[0]) == id_combo[1]


def test_deterministic_uuid_srs_namespaces():
    deterministic_uuid_1 = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.srs_records_holdingsrecord,
        1,
    )
    deterministic_uuid_2 = FolioUUID(
        made_up_okapi_url,
        FOLIONamespaces.srs_records_bib,
        1,
    )
    assert str(deterministic_uuid_2) != str(deterministic_uuid_1)



import enum


class FOLIONamespaces(enum.Enum):
    """
    ENUM for FOLIO Objects. Helping the FolioUUID to create
    unique deterministic uuids within a folio tenant
    """

    # Inventory (0-2, 14)
    holdings = 0
    items = 1
    instances = 2
    authorities = 14

    # SRS (3-5, 16-18)
    srs_records_bib = 3
    srs_records_holdingsrecord = 16
    srs_records_auth = 17
    srs_records_edifact = 18
    raw_records = 4
    parsed_records = 5

    # Circulation (6-7)
    loans = 6
    requests = 7

    # Users (8-9, 23)
    users = 8
    permissions_users = 9
    request_preference = 24

    # Acquisitions (10-12, 15)
    orders = 10
    po_lines = 11
    organizations = 12
    edifact = 15

    # ERM
    # ERM Does not honor generated UUIDs

    # Courses (19-22)
    course = 19
    course_listing = 20
    instructor = 21
    reserve = 22

    # Other
    note = 23
    other = 13



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['folio_uuid', 'folio_uuid.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'folio-uuid',
    'version': '0.2.7',
    'description': 'A library for generating predictive uuids for FOLIO data migrations',
    'long_description': '# folio_uuid\nA python module for creating deterministic UUIDs (UUID v5) outside of FOLIO when migrating data.\n\n# Installation\nThe module is uploaded to pypi. Just do    \n\n\tpip install folio-uuid\n\t\nor   \n\n\tpipenv install folio-uuid      \n\n# Overview\nThe UUIDs (v5) are contstructed in the following way:\n* The namespace is the same for all "Folio UUIDs": **8405ae4d-b315-42e1-918a-d1919900cf3f**\n* The name is contstructed by the following parts, delimited by a colon (**:**)\n\t* **OKAPI_URL** This should be the full OKAPI Url including https. **Example:** *https://okapi-bugfest-juniper.folio.ebsco.com*\n\t* **OBJECT_TYPE_NAME** This should be the name of the type of object that the ID is generated for. In plural. the *file folio_namespaces.py* in this repo has a complete list of the ones currently in use. **Example:** *items*\n\t* **LEGACY_IDENTIFIER** This should be the legacy identifier comming from the source system. The library will perform some normalization* of the identifier if it is a Sierra/Millennium identifier. **Example:** *i3696836*\n\n\\* The normalization strips away any dots (.), check digits and campus codes from the identifiers\n\n# Tests/Examples\n* The namespace is *8405ae4d-b315-42e1-918a-d1919900cf3f*\n* The name, constructed as *OKAPI_URL:OBJECT_TYPE_NAME:LEGACY_IDENTIFIER* would become  *https://okapi-bugfest-juniper.folio.ebsco.com:items:i3696836*\n* The resulting UUID then becomes *9647225d-d8e9-530d-b8cc-52a53be14e26*\n\n# Bash/linux example\n![image](https://user-images.githubusercontent.com/1894384/141293255-a692c61f-4b80-4748-8187-b8bdabe9befa.png)\n\n\tuuidgen --sha1 -n 8405ae4d-b315-42e1-918a-d1919900cf3f -N https://okapi-bugfest-juniper.folio.ebsco.com:items:i3696836\nTo install uuidgen on a apt-enabled Linux distribution, use   \n\n\tsudo apt-get install uuid-runtime\n\n# Python Example\n\tdef test_deterministic_uuid_generation_holdings():\n\t    deterministic_uuid = FolioUUID(\n\t\t"https://okapi-bugfest-juniper.folio.ebsco.com",\n\t\tFOLIONamespaces.holdings,\n\t\t"000000167",\n\t    )\n\t    assert "a0b4c8a2-01fd-50fd-8158-81bd551412a0" == str(deterministic_uuid)\n\t    \n\t    \n# References\nWikipedia has an [article on UUID version 5](https://en.wikipedia.org/wiki/Universally_unique_identifier#Versions_3_and_5_(namespace_name-based))\n\nThere are many browser-based tools to create singe UUIDs v5. [UUIDTools](https://www.uuidtools.com/v5) is one of them.\n',
    'author': 'Theodor Tolstoy',
    'author_email': 'github.teddes@tolstoy.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/FOLIO-FSE/folio_uuid',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
